package org.apache.bcel.verifier.statics;

package .tmp;

import org.apache.bcel.classfile.ClassFormatException;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantFieldref;
import org.apache.bcel.classfile.ConstantInterfaceMethodref;
import org.apache.bcel.classfile.ConstantMethodref;
import org.apache.bcel.classfile.ConstantNameAndType;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.EmptyVisitor;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.generic.Type;
import org.apache.bcel.verifier.exc.ClassConstraintException;
import org.apache.bcel.verifier.statics.Pass2Verifier;

final class FAMRAV_Visitor extends EmptyVisitor {
  private final ConstantPool cp;
  
  private FAMRAV_Visitor(JavaClass jc) {
    this.cp = jc.getConstantPool();
  }
  
  public void visitConstantFieldref(ConstantFieldref obj) {
    if (obj.getTag() != 9)
      throw new ClassConstraintException("ConstantFieldref '" + Pass2Verifier.access$000(obj) + "' has wrong tag!"); 
    int nameAndTypeIndex = obj.getNameAndTypeIndex();
    ConstantNameAndType cnat = (ConstantNameAndType)this.cp.getConstant(nameAndTypeIndex);
    String name = ((ConstantUtf8)this.cp.getConstant(cnat.getNameIndex())).getBytes();
    if (!Pass2Verifier.access$400(name))
      throw new ClassConstraintException("Invalid field name '" + name + "' referenced by '" + Pass2Verifier.access$000(obj) + "'."); 
    int classIndex = obj.getClassIndex();
    ConstantClass cc = (ConstantClass)this.cp.getConstant(classIndex);
    String className = ((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes();
    if (!Pass2Verifier.access$600(className))
      throw new ClassConstraintException("Illegal class name '" + className + "' used by '" + Pass2Verifier.access$000(obj) + "'."); 
    String sig = ((ConstantUtf8)this.cp.getConstant(cnat.getSignatureIndex())).getBytes();
    try {
      Type.getType(sig);
    } catch (ClassFormatException cfe) {
      throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.access$000(obj) + "'.", cfe);
    } 
  }
  
  public void visitConstantInterfaceMethodref(ConstantInterfaceMethodref obj) {
    if (obj.getTag() != 11)
      throw new ClassConstraintException("ConstantInterfaceMethodref '" + Pass2Verifier.access$000(obj) + "' has wrong tag!"); 
    int nameAndTypeIndex = obj.getNameAndTypeIndex();
    ConstantNameAndType cnat = (ConstantNameAndType)this.cp.getConstant(nameAndTypeIndex);
    String name = ((ConstantUtf8)this.cp.getConstant(cnat.getNameIndex())).getBytes();
    if (!Pass2Verifier.access$700(name))
      throw new ClassConstraintException("Invalid (interface) method name '" + name + "' referenced by '" + Pass2Verifier.access$000(obj) + "'."); 
    int classIndex = obj.getClassIndex();
    ConstantClass cc = (ConstantClass)this.cp.getConstant(classIndex);
    String className = ((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes();
    if (!Pass2Verifier.access$600(className))
      throw new ClassConstraintException("Illegal class name '" + className + "' used by '" + Pass2Verifier.access$000(obj) + "'."); 
    String sig = ((ConstantUtf8)this.cp.getConstant(cnat.getSignatureIndex())).getBytes();
    try {
      Type t = Type.getReturnType(sig);
      if (name.equals("<clinit>") && t != Type.VOID)
        Pass2Verifier.this.addMessage("Class or interface initialization method '<clinit>' usually has VOID return type instead of '" + t + "'. Note this is really not a requirement of The Java Virtual Machine Specification, Second Edition."); 
    } catch (ClassFormatException cfe) {
      throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.access$000(obj) + "'.", cfe);
    } 
  }
  
  public void visitConstantMethodref(ConstantMethodref obj) {
    if (obj.getTag() != 10)
      throw new ClassConstraintException("ConstantMethodref '" + Pass2Verifier.access$000(obj) + "' has wrong tag!"); 
    int nameAndTypeIndex = obj.getNameAndTypeIndex();
    ConstantNameAndType cnat = (ConstantNameAndType)this.cp.getConstant(nameAndTypeIndex);
    String name = ((ConstantUtf8)this.cp.getConstant(cnat.getNameIndex())).getBytes();
    if (!Pass2Verifier.access$800(name))
      throw new ClassConstraintException("Invalid (non-interface) method name '" + name + "' referenced by '" + Pass2Verifier.access$000(obj) + "'."); 
    int classIndex = obj.getClassIndex();
    ConstantClass cc = (ConstantClass)this.cp.getConstant(classIndex);
    String className = ((ConstantUtf8)this.cp.getConstant(cc.getNameIndex())).getBytes();
    if (!Pass2Verifier.access$600(className))
      throw new ClassConstraintException("Illegal class name '" + className + "' used by '" + Pass2Verifier.access$000(obj) + "'."); 
    String sig = ((ConstantUtf8)this.cp.getConstant(cnat.getSignatureIndex())).getBytes();
    try {
      Type t = Type.getReturnType(sig);
      if (name.equals("<init>") && t != Type.VOID)
        throw new ClassConstraintException("Instance initialization method must have VOID return type."); 
    } catch (ClassFormatException cfe) {
      throw new ClassConstraintException("Illegal descriptor (==signature) '" + sig + "' used by '" + Pass2Verifier.access$000(obj) + "'.", cfe);
    } 
  }
}
